################################################################################
#                                                                              #
#         Supplement to Brinks et al (2020): Epidemiological measures          #
#      during the SARS-CoV-2-outbreak: bias by incomplete case-detection       #
#                                                                              #
#                Script for use with the statistical software R                #
#                                                                              #
################################################################################

rm(list=ls(all=TRUE))

######################
#
#     Input data     #
#
######################

# grid size (in days, 1/24 = 1 hour)
dh      <- 1/24  # 0.02 takes two minutes, gets very slow below 0.02

gam_    <- 0.25  # rate gamma
S0      <- 1e5   # initial number of susceptibles

t.max   <- 60
tau.max <- 30

# time scales
t_      <- seq(0, t.max,   by = dh)  # period
tau_    <- seq(0, tau.max, by = dh)  # infection ages


# aux functions
fct_g <- function(tt){
   return(ifelse(tt <= 1, 0, gam_))
}

fct_G <- function(tt){
   return(ifelse(tt <= 1, 0, gam_*(tt - 1)))
}

fct_i0 <- function(tau){
   i0_ <- 20
   return(i0_*exp(-fct_G(tau)))
}

# initial distribution i(t0)
i0_ <- fct_i0(tau_)


# transmission rate
fct_beta <- function(t, tau){
   tw_ <- dgamma(tau, shape = 2, rate = gam_)

   tt_ <- c(  25,   30)
   vv_ <- c(   1,   .5)
   tv_ <- approx(tt_, vv_, xout = t, rule = 2)$y
   return(tv_*tw_*6e-5)
}

######################
#     Simulating 
#    the epidemics
######################


## (Lexis) matrix to store density of infected
imat    <- matrix(data = NA,    nrow = length(t_), ncol = length(tau_))

# function for filling up imat 
fillDiag <- function(rs, cs, sval){
   # fills imat along the diagonal starting
   # at position (rs, cs) with value sval
   
   # calc length of diagonal
   d.len <- 1 + min(nrow(imat) - rs, ncol(imat) - cs)
   imat[rs, cs] <<- sval
   if(d.len >= 2){
      for(ii in 2:d.len){
         imat[rs + ii - 1, cs + ii - 1] <<- imat[rs + ii - 2, cs + ii - 2] * 
                                            exp(-dh*fct_g(tau_[cs + ii - 1]))
      }
   }
}

# fill imat above main diagonale
# init first row with i0 = i(0, .)
for(ii in 1:ncol(imat)){
   fillDiag(1, ii, i0_[ii])
}

# Susceptibles
S          <- numeric(length(t_))
S[1]       <- S0

# force of infection
lambda_    <- numeric(length(t_))
lambda_[1] <- dh * sum(fct_beta(t_[1], tau_) * imat[1,]) # midpoint rule to integrate beta * i

for(rnr in 2:nrow(imat)){
   i_ <- lambda_[rnr - 1] * S[rnr - 1]
   fillDiag(rnr, 1, i_)
   lambda_[rnr] <- dh * sum(fct_beta(t_[rnr], tau_) * imat[rnr,]) # again midpoint rule
   S[rnr] <- S[rnr-1] * exp(-dh*lambda_[rnr])
}

# infected: midpoint rule for calculating I(t)
I <- dh*rowSums(imat)


# analytical calcution of the instantaneous reproduction number R.eff
anaR    <- numeric(length(t_))
Gam.tau <- exp(-fct_G(tau_))
for(ii in 1:length(t_)){
   anaR[ii] <- S[ii] * dh * sum(fct_beta(t_[ii], tau_) * Gam.tau)
}

############ simulation completed ###############


######################
#
# Mimic scenarios
#  of case-detection
#
######################


# Cases are reported on a daily basis -> aggregate by days
myT <- 1:t.max

# True incident cases
F_t <- dh*rowSums(matrix(data = (S*lambda_)[-1], ncol = 1/dh, byrow = TRUE))

# function to empirically estimte R.eff
fct_estReff <- function(vC){
   w_  <- c(rep(0, 3), .6, rep(1, 5), .8)
   genT <- 9+1 # +1 as R vectors start with 1
   res <- rep(NA, length(vC))
   for(ii in (genT+1):length(vC)){
      res[ii] <- (sum(w_)) * vC[ii]/
                    (  
                       w_[ 1] * vC[ii-(genT+0)] + w_[ 2] * vC[ii-(genT-1)] + 
                       w_[ 3] * vC[ii-(genT-2)] + w_[ 4] * vC[ii-(genT-3)] +
                       w_[ 5] * vC[ii-(genT-4)] + w_[ 6] * vC[ii-(genT-5)] +
                       w_[ 7] * vC[ii-(genT-6)] + w_[ 8] * vC[ii-(genT-7)] +
                       w_[ 9] * vC[ii-(genT-8)] + w_[10] * vC[ii-(genT-9)] 
                    )
   }
   return(res)
}
R.eff <- fct_estReff(F_t)

# function to estimate doubling time
fct_doublingTime <- function(t_, vS_, windowWidth = 10){
   nn <- length(vS_)
   T2 <- rep(NA, nn)
   sIdx <- 1
   for(ii in windowWidth:nn){

      # linear fit
      mfit <- summary(lm(log(vS_) ~ t_, subset = (t_[sIdx] <= t_ & t_ <= ii)))
      T2[ii] <- log(2)/mfit$coeff[2,1]
      sIdx <- sIdx + 1
   }
   return(T2)
}


### scenarios of case-detection

nSc      <- 4 # number of scenarios
CDR      <- matrix(data = NA, nrow = nSc, ncol = length(myT))
CDR[1, ] <- rep(0.1, length(myT))
CDR[2, ] <- rep(0.5, length(myT))
CDR[3, ] <- 0.8 - (0.02*(myT-40))^2
CDR[4, ] <- 0.9 - (5e-4*myT*(60 - myT))^2


# Observed incident cases F^o_t in the different scenarios
Fo_t <- CDR * matrix(data = F_t, nrow = nSc, ncol = length(myT), byrow = TRUE)

#########################
#
#   evaluation of bias
#  -> data for Table 1
#      of main text
#
#########################

# (1) cumulative case count (CCC)
CCC   <- cumsum(F_t)
CCC.o <- t(apply(Fo_t, FUN = cumsum, MARGIN = 1))
t(round(100*(t(CCC.o[,c(15, 30, 45, 60)])/CCC[c(15, 30, 45, 60)] - 1)))

# (2) incidence per inhabitant
t(round(100*(t(Fo_t[,c(15, 30, 45, 60)]) / F_t[c(15, 30, 45, 60)] - 1)))

# (3) effective reproduction number
Reff.o <- t(apply(Fo_t, FUN = fct_estReff, MARGIN = 1))
t(round(100*(t(Reff.o[,c(15, 30, 45, 60)]) / R.eff[c(15, 30, 45, 60)] - 1)))

# (4) doubling time
DT   <- fct_doublingTime(myT, CCC)
DT.o <- t(apply(CCC.o, FUN = fct_doublingTime, t_ = myT, MARGIN = 1))
t(round(100*(t(DT.o[,c(15, 30, 45, 60)]) / DT[c(15, 30, 45, 60)] - 1)))


